/**********************************************************************
 * @file HAL_Settings.h
 * @brief Global hardware configuration settings for the project
 *        (include in a lot of source files)
 * @version 0.0.1
 ***********************************************************************
 * @section License
 * <b>(C) Copyright 2017 Falk Richter, falk.richter at yandex.com</b>
 ***********************************************************************
 *
 * This file is licensed under the GPLv3.
 *
 **********************************************************************/

#ifndef _HAL_SETTINGS_H_
#define	_HAL_SETTINGS_H_ 

// to declaration the special function registers (nearly) independent
// of the compiler we need this file
#include <compiler.h>

// define the processor and all special function registers
#include "STC15F2K60S2.h" 

// initialize the hardware abstract layer
extern void HAL_vInit(void);

//----------------------------------------------------------------------- 
// periphery definitions 
//-----------------------------------------------------------------------


// Clock frequency
// ---------------------------------------- 
// microprocessor main clock frequency (crystal clock or RC-generator)
// if you are using an UART you should take a frequency which is a 
// (full integer) multiple of the serial baudrate (e.g. 11059200MHz:
// 11059200 / 9600 = 1152) to reach a good baudrate tolerance
// value is defined in the HAL_Settings.c file
extern const uint32_t UC_u32OscFreq;


// UART1 
// ---------------------------------------- 
// Uart1 baudrate
// value is defined in the HAL_Settings.c file 
extern const uint32_t UART1_u32Baudrate;


// uC processor flag for the low voltage detection
// ---------------------------------------- 
extern const uint8_t HAL_u8LvdFlag;

// Infrared receiver 
// ---------------------------------------- 
// input pin of the ir receiver
SBIT(PIN_IR_RX, P3_ADR, 7);


// LCD1602 driver (2x16 characters display)
// ---------------------------------------- 
// display power supply on / off
SBIT(LCD1602A_PWR, P4_ADR, 5); // Pin P4.5
// port for the data transfer
SFR(LCD1602A_PORT, P2_ADR); // Port 2 P2.[0...7]
// enable signal (high -> low slope)
SBIT(LCD1602A_ENA, P4_ADR, 4); // Pin P4.4
// read / write signal (high: read, low: write)
SBIT(LCD1602A_RW, P4_ADR, 2); // Pin P4.2
// register select signal (high: data input, low: instruction input)
SBIT(LCD1602A_RS, P4_ADR, 1); // Pin P4.1 


// EEPROM interface
// ----------------------------------------
// chip select pin
SBIT(EEPROM_CS, P1_ADR, 2);
// uC: MOSI
SBIT(EEPROM_MO, P1_ADR, 3);
// uC: MISO
SBIT(EEPROM_MI, P1_ADR, 4);
// clock pin
SBIT(EEPROM_CLK, P1_ADR, 5); 

/* valid for 25LC010 EEPROM
// write protection pin
SBIT(EEPROM_WP, P1_ADR, 1);
// all other pins must not declared here
// SCLK: P1.5
// MISO: P1.4
// MOSI: P1.3
*/

// Potentiometer
// ----------------------------------------
// parallel relay port (only 7 bits are used)
SFR(POTI_PORT, P0_ADR); // Port 0 P0.[0...7]


// Input selector
// ----------------------------------------
// channel 4
SBIT(INPUTSEL_CH4, P3_ADR, 3); // Pin P3.3
// channel 3
SBIT(INPUTSEL_CH3, P3_ADR, 4); // Pin P3.4
// channel 2
SBIT(INPUTSEL_CH2, P3_ADR, 5); // Pin P3.5
// channel 1
SBIT(INPUTSEL_CH1, P3_ADR, 6); // Pin P3.6


// ON command for 12V power supply
// ----------------------------------------
SBIT(PWR_ON, P3_ADR, 2); // Pin P3.2

// switch on/off plop protection
// ---------------------------------------- 
SBIT(PLOP_PROTECT, P1_ADR, 1); // Pin P1.1 

// the rotary encoder
// ----------------------------------------
// the switch pin on the encoder
SBIT(ROT_SW, P5_ADR, 5); // Pin P5.5
// the leading signal of the encoder
SBIT(ROT_CLK, P1_ADR, 0); // Pin P1.0
// the second signal of the encoder
SBIT(ROT_DT, P5_ADR, 4); // Pin P5.4

#endif

