#ifndef __STC15F2K60S2_H_
#define __STC15F2K60S2_H_

///////////////////////////////////////////////
// Note: STC15W4K32S4 series of chips, after power-on all the PWM-related IO port are
// High-impedance state, these ports should be set to quasi-bidirectional or strong push-pull mode can be used normally
// Related IO: P0.6 / P0.7 / P1.6 / P1.7 / P2.1 / P2.2
// P2.3 / P2.7 / P3.7 / P4.2 / P4.4 / P4.5
///////////////////////////////////////////////

///////////////////////////////////////////////

// Contains the header file, do not have to include "REG51.H"

// Kernel Special Function Registers // Reset Value Description
SFR(ACC, 0xE0); // 0000,0000 Accumulator Accumulator
SFR(B, 0xF0); // 0000,0000 B register
#define PSW_ADR 0xD0
SFR(PSW, PSW_ADR); // 0000,0000 program status word
SBIT(CY, PSW_ADR, 7);
SBIT(AC, PSW_ADR, 6);
SBIT(F0, PSW_ADR, 5);
SBIT(RS1, PSW_ADR, 4);
SBIT(RS0, PSW_ADR, 3);
SBIT(OV, PSW_ADR, 2);
SBIT(P, PSW_ADR, 0);
SFR(SP, 0x81); // 0000,0111 Stack pointer
SFR(DPL, 0x82); // 0000,0000 data pointer low byte
SFR(DPH, 0x83); // 0000,0000 data pointer high byte

// I / O port special function register
#define P0_ADR 0x80
SFR(P0, P0_ADR); // 1111,1111 port 0
SBIT(P00, P0_ADR, 0);
SBIT(P01, P0_ADR, 1);
SBIT(P02, P0_ADR, 2);
SBIT(P03, P0_ADR, 3);
SBIT(P04, P0_ADR, 4);
SBIT(P05, P0_ADR, 5);
SBIT(P06, P0_ADR, 6);
SBIT(P07, P0_ADR, 7);
#define P1_ADR 0x90
SFR(P1, P1_ADR); // 1111,1111 port 1
SBIT(P10, P1_ADR, 0);
SBIT(P11, P1_ADR, 1);
SBIT(P12, P1_ADR, 2);
SBIT(P13, P1_ADR, 3);
SBIT(P14, P1_ADR, 4);
SBIT(P15, P1_ADR, 5);
SBIT(P16, P1_ADR, 6);
SBIT(P17, P1_ADR, 7);
#define P2_ADR 0xA0
SFR(P2, P2_ADR); // 1111,1111 port 2
SBIT(P20, P2_ADR, 0);
SBIT(P21, P2_ADR, 1);
SBIT(P22, P2_ADR, 2);
SBIT(P23, P2_ADR, 3);
SBIT(P24, P2_ADR, 4);
SBIT(P25, P2_ADR, 5);
SBIT(P26, P2_ADR, 6);
SBIT(P27, P2_ADR, 7);
#define P3_ADR 0xB0
SFR(P3, P3_ADR); // 1111,1111 port 3
SBIT(P30, P3_ADR, 0);
SBIT(P31, P3_ADR, 1);
SBIT(P32, P3_ADR, 2);
SBIT(P33, P3_ADR, 3);
SBIT(P34, P3_ADR, 4);
SBIT(P35, P3_ADR, 5);
SBIT(P36, P3_ADR, 6);
SBIT(P37, P3_ADR, 7);
#define P4_ADR 0xC0
SFR(P4, P4_ADR); // 1111,1111 port 4
SBIT(P40, P4_ADR, 0);
SBIT(P41, P4_ADR, 1);
SBIT(P42, P4_ADR, 2);
SBIT(P43, P4_ADR, 3);
SBIT(P44, P4_ADR, 4);
SBIT(P45, P4_ADR, 5);
SBIT(P46, P4_ADR, 6);
SBIT(P47, P4_ADR, 7);
#define P5_ADR 0xC8
SFR(P5, P5_ADR); // xxxx, 1111 port 5
SBIT(P50, P5_ADR, 0);
SBIT(P51, P5_ADR, 1);
SBIT(P52, P5_ADR, 2);
SBIT(P53, P5_ADR, 3);
SBIT(P54, P5_ADR, 4);
SBIT(P55, P5_ADR, 5);
SBIT(P56, P5_ADR, 6);
SBIT(P57, P5_ADR, 7);
#define P6_ADR 0xE8
SFR(P6, P6_ADR); // 0000,0000 port 6
SBIT(P60, P6_ADR, 0);
SBIT(P61, P6_ADR, 1);
SBIT(P62, P6_ADR, 2);
SBIT(P63, P6_ADR, 3);
SBIT(P64, P6_ADR, 4);
SBIT(P65, P6_ADR, 5);
SBIT(P66, P6_ADR, 6);
SBIT(P67, P6_ADR, 7);
#define P7_ADR 0xF8
SFR(P7, P7_ADR); // 0000,0000 port 7
SBIT(P70, P7_ADR, 0);
SBIT(P71, P7_ADR, 1);
SBIT(P72, P7_ADR, 2);
SBIT(P73, P7_ADR, 3);
SBIT(P74, P7_ADR, 4);
SBIT(P75, P7_ADR, 5);
SBIT(P76, P7_ADR, 6);
SBIT(P77, P7_ADR, 7);

SFR(P0M0, 0x94); // 0000,0000 port 0 mode register 0
SFR(P0M1, 0x93); // 0000,0000 port 0 mode register 1
SFR(P1M0, 0x92); // 0000,0000 port 1 mode register 0
SFR(P1M1, 0x91); // 0000,0000 port 1 mode register 1
SFR(P2M0, 0x96); // 0000,0000 port 2 mode register 0
SFR(P2M1, 0x95); // 0000,0000 port 2 mode register 1
SFR(P3M0, 0xB2); // 0000,0000 port 3 mode register 0
SFR(P3M1, 0xB1); // 0000,0000 port 3 mode register 1
SFR(P4M0, 0xB4); // 0000,0000 port 4 mode register 0
SFR(P4M1, 0xB3); // 0000,0000 port 4 mode register 1
SFR(P5M0, 0xCA); // 0000,0000 port 5 mode register 0
SFR(P5M1, 0xC9); // 0000,0000 port 5 mode register 1
SFR(P6M0, 0xCC); // 0000,0000 port 6 mode register 0
SFR(P6M1, 0xCB); // 0000,0000 port 6 mode register 1
SFR(P7M0, 0xE2); // 0000,0000 port 7 mode register 0
SFR(P7M1, 0xE1); // 0000,0000 port 7 mode register 1

// System Management Special Function Registers
SFR(PCON, 0x87); // 0001,0000 power control register
SFR(AUXR, 0x8E); // 0000,0000 Auxiliary register
SFR(AUXR1, 0xA2); // 0000,0000 Auxiliary register 1
SFR(P_SW1, 0xA2); // 0000,0000 Peripheral port switch register 1
SFR(CLK_DIV, 0x97); // 0000,0000 clock divider control register
SFR(BUS_SPEED, 0xA1); // xx10, x011 bus speed control register
SFR(P1ASF, 0x9D); // 0000,0000 port 1 analog function configuration register
SFR(P_SW2, 0xBA); // 0xxx, x000 Peripheral port switch register

// interrupt the special function register
#define IE_ADR 0xA8
SFR(IE, IE_ADR); // 0000,0000 interrupt control register
SBIT(EA, IE_ADR, 7);
SBIT(ELVD, IE_ADR, 6);
SBIT(EADC, IE_ADR, 5);
SBIT(ES, IE_ADR, 4);
SBIT(ET1, IE_ADR, 3);
SBIT(EX1, IE_ADR, 2);
SBIT(ET0, IE_ADR, 1);
SBIT(EX0, IE_ADR, 0);
#define IP_ADR 0xB8
SFR(IP, IP_ADR); // 0000,0000 interrupt priority register
SBIT(PPCA, IP_ADR, 7);
SBIT(PLVD, IP_ADR, 6);
SBIT(PADC, IP_ADR, 5);
SBIT(PS, IP_ADR, 4);
SBIT(PT1, IP_ADR, 3);
SBIT(PX1, IP_ADR, 2);
SBIT(PT0, IP_ADR, 1);
SBIT(PX0, IP_ADR, 0);
SFR(IE2, 0xAF); // 0000,0000 interrupt control register 2
SFR(IP2, 0xB5); // xxxx, xx00 interrupt priority register 2
SFR(INT_CLKO, 0x8F); // 0000,0000 External Interrupt and Clock Output Control Register

// timer special function register
#define TCON_ADR 0x88
SFR(TCON, TCON_ADR); // 0000,0000 T0 / T1 control register
SBIT(TF1, TCON_ADR, 7);
SBIT(TR1, TCON_ADR, 6);
SBIT(TF0, TCON_ADR, 5);
SBIT(TR0, TCON_ADR, 4);
SBIT(IE1, TCON_ADR, 3);
SBIT(IT1, TCON_ADR, 2);
SBIT(IE0, TCON_ADR, 1);
SBIT(IT0, TCON_ADR, 0);
SFR(TMOD, 0x89); // 0000,0000 T0 / T1 mode register
SFR(TL0, 0x8A); // 0000,0000 T0 low byte
SFR(TL1, 0x8B); // 0000,0000 T1 low byte
SFR(TH0, 0x8C); // 0000,0000 T0 high byte
SFR(TH1, 0x8D); // 0000,0000 T1 high byte
SFR(T4T3M, 0xD1); // 0000,0000 T3 / T4 mode register
SFR(T3T4M, 0xD1); // 0000,0000 T3 / T4 mode register
SFR(T4H, 0xD2); // 0000,0000 T4 high byte
SFR(T4L, 0xD3); // 0000,0000 T4 low byte
SFR(T3H, 0xD4); // 0000,0000 T3 high byte
SFR(T3L, 0xD5); // 0000,0000 T3 low byte
SFR(T2H, 0xD6); // 0000,0000 T2 high byte
SFR(T2L, 0xD7); // 0000,0000 T2 low byte
SFR(TH4, 0xD2); // 0000,0000 T4 high byte
SFR(TL4, 0xD3); // 0000,0000 T4 low byte
SFR(TH3, 0xD4); // 0000,0000 T3 high byte
SFR(TL3, 0xD5); // 0000,0000 T3 low byte
SFR(TH2, 0xD6); // 0000,0000 T2 high byte
SFR(TL2, 0xD7); // 0000,0000 T2 low byte

SFR(WKTCL, 0xAA); // 0000,0000 power-down wakeup timer low byte
SFR(WKTCH, 0xAB); // 0000,0000 power-down wakeup timer high byte
SFR(WDT_CONTR, 0xC1); // 0000,0000 Watchdog control register

// serial port special function register
#define SCON_ADR 0x98 
SFR(SCON, SCON_ADR); // 0000,0000 serial port 1 control register
SBIT(SM0, SCON_ADR, 7);
SBIT(SM1, SCON_ADR, 6);
SBIT(SM2, SCON_ADR, 5);
SBIT(REN, SCON_ADR, 4);
SBIT(TB8, SCON_ADR, 3);
SBIT(RB8, SCON_ADR, 2);
SBIT(TI, SCON_ADR, 1);
SBIT(RI, SCON_ADR, 0);
SFR(SBUF, 0x99); // xxxx, xxxx Serial 1 data register
SFR(S2CON, 0x9A); // 0000,0000 serial port 2 control register
SFR(S2BUF, 0x9B); // xxxx, xxxx serial port 2 data register
SFR(S3CON, 0xAC); // 0000,0000 serial port 3 control register
SFR(S3BUF, 0xAD); // xxxx, xxxx serial port 3 data register
SFR(S4CON, 0x84); // 0000,0000 serial port 4 control register
SFR(S4BUF, 0x85); // xxxx, xxxx serial port 4 data register
SFR(SADDR, 0xA9); // 0000,0000 Slave Address Register
SFR(SADEN, 0xB9); // 0000,0000 Slave Address Mask Register

// ADC special function register
SFR(ADC_CONTR, 0xBC); // 0000,0000 A / D conversion control register
SFR(ADC_RES, 0xBD); // 0000,0000 A / D conversion result is high 8 bits
SFR(ADC_RESL, 0xBE); // 0000,0000 A / D conversion result is low 2 bits

// SPI special function register
SFR(SPSTAT, 0xCD); // 00xx, xxxx SPI status register
SFR(SPCTL, 0xCE); // 0000,0100 SPI control register
SFR(SPDAT, 0xCF); // // 0000,0000 SPI data register

// IAP / ISP special function register
SFR(IAP_DATA, 0xC2); // 0000,0000 EEPROM data register
SFR(IAP_ADDRH, 0xC3); // 0000,0000 EEPROM address high byte
SFR(IAP_ADDRL, 0xC4); // 0000,0000 EEPROM address byte
SFR(IAP_CMD, 0xC5); // xxxx, xx00 EEPROM command register
SFR(IAP_TRIG, 0xC6); // 0000,0000 EEPRPM Command Trigger Register
SFR(IAP_CONTR, 0xC7); // 0000, x000 EEPROM control register

// PCA / PWM special function register
#define CCON_ADR 0xD8
SFR(CCON, CCON_ADR); // 00xx, xx00 PCA control register
SBIT(CF, CCON_ADR, 7);
SBIT(CR, CCON_ADR, 6);
SBIT(CCF2, CCON_ADR, 2);
SBIT(CCF1, CCON_ADR, 1);
SBIT(CCF0, CCON_ADR, 0);
SFR(CMOD, 0xD9); // 0xxx, x000 PCA operating mode register
SFR(CL, 0xE9); // 0000,0000 PCA counter low byte
SFR(CH, 0xF9); // 0000,0000 PCA counter high byte
SFR(CCAPM0, 0xDA); // 0000,0000 PCA module 0 PWM register
SFR(CCAPM1, 0xDB); // 0000,0000 PCA module 1 PWM register
SFR(CCAPM2, 0xDC); // 0000,0000 PCA module 2 PWM register
SFR(CCAP0L, 0xEA); // 0000,0000 PCA module 0 capture / compare register low byte
SFR(CCAP1L, 0xEB); // 0000,0000 PCA module 1 capture / compare register low byte
SFR(CCAP2L, 0xEC); // 0000,0000 PCA module 2 capture / compare register low byte
SFR(PCA_PWM0, 0xF2); // xxxx, xx00 PWM register for PCA module 0
SFR(PCA_PWM1, 0xF3); // xxxx, xx00 PWM register for PCA module 1
SFR(PCA_PWM2, 0xF4); // xxxx, xx00 PWM register for PCA module 1
SFR(CCAP0H, 0xFA); // 0000,0000 PCA module 0 capture / compare register high byte
SFR(CCAP1H, 0xFB); // 0000,0000 PCA module 1 capture / compare register high byte
SFR(CCAP2H, 0xFC); // 0000,0000 PCA module 2 capture / compare register high byte

// comparator special function register
SFR(CMPCR1, 0xE6); // 0000,0000 comparator control register 1
SFR(CMPCR2, 0xE7); // 0000,0000 comparator control register 2

// Enhanced PWM Waveform Generator Special Function Registers
SFR(PWMCFG, 0xf1); // x000,0000 PWM configuration register
SFR(PWMCR, 0xf5); // 0000,0000 PWM control register
SFR(PWMIF, 0xf6); // x000,0000 PWM interrupt flag register
SFR(PWMFDCR, 0xf7); // xx00,0000 PWM External Exception Detection Control Register

// The following special function registers are located in the extended RAM area
// Access to these registers, the need to first P_SW2 BIT7 set to 1, can be normal read and write
#define PWMC (* (unsigned int volatile xdata *) 0xfff0)
#define PWMCH (* (unsigned char volatile xdata *) 0xfff0)
#define PWMCL (* (unsigned char volatile xdata *) 0xfff1)
#define PWMCKS (* (unsigned char volatile xdata *) 0xfff2)
#define PWM2T1 (* (unsigned int volatile xdata *) 0xff00)
#define PWM2T1H (* (unsigned char volatile xdata *) 0xff00)
#define PWM2T1L (* (unsigned char volatile xdata *) 0xff01)
#define PWM2T2 (* (unsigned int volatile xdata *) 0xff02)
#define PWM2T2H (* (unsigned char volatile xdata *) 0xff02)
#define PWM2T2L (* (unsigned char volatile xdata *) 0xff03)
#define PWM2CR (* (unsigned char volatile xdata *) 0xff04)
#define PWM3T1 (* (unsigned int volatile xdata *) 0xff10)
#define PWM3T1H (* (unsigned char volatile xdata *) 0xff10)
#define PWM3T1L (* (unsigned char volatile xdata *) 0xff11)
#define PWM3T2 (* (unsigned int volatile xdata *) 0xff12)
#define PWM3T2H (* (unsigned char volatile xdata *) 0xff12)
#define PWM3T2L (* (unsigned char volatile xdata *) 0xff13)
#define PWM3CR (* (unsigned char volatile xdata *) 0xff14)
#define PWM4T1 (* (unsigned int volatile xdata *) 0xff20)
#define PWM4T1H (* (unsigned char volatile xdata *) 0xff20)
#define PWM4T1L (* (unsigned char volatile xdata *) 0xff21)
#define PWM4T2 (* (unsigned int volatile xdata *) 0xff22)
#define PWM4T2H (* (unsigned char volatile xdata *) 0xff22)
#define PWM4T2L (* (unsigned char volatile xdata *) 0xff23)
#define PWM4CR (* (unsigned char volatile xdata *) 0xff24)
#define PWM5T1 (* (unsigned int volatile xdata *) 0xff30)
#define PWM5T1H (* (unsigned char volatile xdata *) 0xff30)
#define PWM5T1L (* (unsigned char volatile xdata *) 0xff31)
#define PWM5T2 (* (unsigned int volatile xdata *) 0xff32)
#define PWM5T2H (* (unsigned char volatile xdata *) 0xff32)
#define PWM5T2L (* (unsigned char volatile xdata *) 0xff33)
#define PWM5CR (* (unsigned char volatile xdata *) 0xff34)
#define PWM6T1 (* (unsigned int volatile xdata *) 0xff40)
#define PWM6T1H (* (unsigned char volatile xdata *) 0xff40)
#define PWM6T1L (* (unsigned char volatile xdata *) 0xff41)
#define PWM6T2 (* (unsigned int volatile xdata *) 0xff42)
#define PWM6T2H (* (unsigned char volatile xdata *) 0xff42)
#define PWM6T2L (* (unsigned char volatile xdata *) 0xff43)
#define PWM6CR (* (unsigned char volatile xdata *) 0xff44)
#define PWM7T1 (* (unsigned int volatile xdata *) 0xff50)
#define PWM7T1H (* (unsigned char volatile xdata *) 0xff50)
#define PWM7T1L (* (unsigned char volatile xdata *) 0xff51)
#define PWM7T2 (* (unsigned int volatile xdata *) 0xff52)
#define PWM7T2H (* (unsigned char volatile xdata *) 0xff52)
#define PWM7T2L (* (unsigned char volatile xdata *) 0xff53)
#define PWM7CR (* (unsigned char volatile xdata *) 0xff54)

///////////////////////////////////////////////

/*   interrupt vector */
#define		INT0_VECTOR		0
#define		TIMER0_VECTOR	1u
#define		INT1_VECTOR		2u
#define		TIMER1_VECTOR	3u
#define		UART1_VECTOR	4u
#define		ADC_VECTOR		5u
#define		LVD_VECTOR		6u
#define		PCA_VECTOR		7u
#define		UART2_VECTOR	8u
#define		SPI_VECTOR		9u
#define		INT2_VECTOR		10u
#define		INT3_VECTOR		11u
#define		TIMER2_VECTOR	12u
#define		INT4_VECTOR		16u
#define		UART3_VECTOR	17u
#define		UART4_VECTOR	18u
#define		TIMER3_VECTOR	19u
#define		TIMER4_VECTOR	20u
#define		CMP_VECTOR		21u

#endif

