/*********************************************************************** 
 * @file RotaryEnc.c
 * @brief Rotary encoder for volume and input control 
 * @version 0.0.1
 ***********************************************************************
 * @section License
 * <b>(C) Copyright 2017 Falk Richter, falk.richter at yandex.com</b>
 ***********************************************************************
 *
 * This file is licensed under the GPLv3.
 *
 **********************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "HAL_Settings.h"
#include "RotaryEnc.h"

// pin A is also sometime called CLK,
// pin B DT

// variable holds the last state of pin A
static uint8_t mg_u8LastPinA;
// variable holds the last state of the switch
static uint8_t mg_u8LastPinSw; 

//----------------------------------------------------------------------
// initialize the rotary switch module
//---------------------------------------------------------------------- 
void ROT_vInit(void) {
  // we initialize the old value with the current value
  mg_u8LastPinA = ROT_CLK;
  mg_u8LastPinSw = ROT_SW;
} 


//----------------------------------------------------------------------
// poll the inputs of the rotary switch
//
// returns:
// 0 - no activity
// 1 - clockwise rotated
// 2 - counter clockwise rotated
// 3 - switch a short time pressed
// 4 - switch a long time pressed
//---------------------------------------------------------------------- 
uint8_t ROT_u8Encoder(void) {
  // the return value
  uint8_t u8ReturnVal = 0;
  // variable to hold the current value of pin A
  uint8_t u8PinA;
  // variable to hold the current value of switch pin 
  uint8_t u8PinSw;
  // variable to count the duration how long the switch was pressed
  static uint16_t u16SwDuration;

  // read pin A
  u8PinA = ROT_CLK;
  // read the switch pin
  u8PinSw = ROT_SW;

  /*
  // check if there was an activity
  if ( u8PinA != mg_u8LastPinA ) {
  // the knob was rotated we look in which direction
  if (u8PinA == ROT_DT ) {
  // the knob was rotated clockwise
  u8ReturnVal = 1u;
  } else {
  // the knob was rotated counter clockwise
  u8ReturnVal = 2u;
  }
  } else {
  // the knob wasn't rotated we check if the knob was pressed
  if ( u8PinSw != mg_u8LastPinSw ) {
  // the knob was pressed
  u8ReturnVal = 3u;
  } 
  }
  */

  // check if there was an activity
  if ( (u8PinA == 0) && (mg_u8LastPinA == 1) ) {
	// the knob was rotated we look in which direction
	if ( ROT_DT == 0 ) {
	  // the knob was rotated clockwise
	  u8ReturnVal = 1u;
	} else {
	  // the knob was rotated counter clockwise
	  u8ReturnVal = 2u;
	}
  } else {
	// the knob wasn't rotated we check if the knob was pressed
	// detect if the knob was just pressed
	if ( (u8PinSw == 0) && (mg_u8LastPinSw == 1) ) {
	  // reset the duration counter
	  u16SwDuration = 0;
	}
	else if ( (u8PinSw == 1) && (mg_u8LastPinSw == 0) ) {
	  // the knob was released
	  if ( u16SwDuration < 10000u ) {
		// the switch was pressed a short time 
		u8ReturnVal = 3u;
	  } else {
		// the switch was pressed a long time 
		u8ReturnVal = 4u;
	  } 
	}
	else if ( u8PinSw == 0 ) {
	  // the knob is held down - we count the duration
	  if ( u16SwDuration < 10000u ) {
		// we avoid overflows
		u16SwDuration++;
	  }
	}
  }
  
  // refresh the storage variables
  mg_u8LastPinA = u8PinA;
  mg_u8LastPinSw = u8PinSw;

  return(u8ReturnVal);
}

