/**********************************************************************
 * HAL_Settings.c
 * Global hardware configuration settings for the project
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

// include the standard integer header
#include <stdint.h>

// include the own header
#include "HAL_Settings.h"

//----------------------------------------------------------------------- 
// definition of project global variables
//-----------------------------------------------------------------------

// microprocessor main clock frequency (crystal clock or RC-generator)
// if you are using an UART you should take a frequency which is a 
// (full integer) multiple of the serial baudrate (e.g. 11059200MHz:
// 11059200 / 9600 = 1152) to reach a good baudrate tolerance
const uint32_t UC_u32OscFreq = 22118400u;

// UART1
// -----

// Uart1 baudrate
const uint32_t UART1_u32Baudrate = 115200u;

// flag for the low voltage detection
const uint8_t HAL_u8LvdFlag = 0x20;

//----------------------------------------------------------------------
// initialize the hardware abstract layer
//---------------------------------------------------------------------- 
void HAL_vInit(void) {

  // Set the operation mode of the ports
  
  // standard is quasi bidirectional
  P0M1 = 0;	P0M0 = 0;
  // the rotary encoder inputs must not have an internal pull-up 
  P1M1 = 0x01; P1M0 = 0;
  P2M1 = 0; P2M0 = 0;
  P3M1 = 0; P3M0 = 0; 
  P4M1 = 0; P4M0 = 0; 
  // the rotary encoder inputs must not have an internal pull-up
  P5M1 = 0x30; P5M0 = 0; 

  // after reset the low voltage detection flag is set -> we clear it
  PCON &= ~HAL_u8LvdFlag;
}

