/***********************************************************************
 * InputSelect.c
 * Selector for the line inputs
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include <stdint.h>
#include "HAL_Settings.h" 
#include "InputSelect.h" 

static uint8_t mg_u8CurrentInput;


//----------------------------------------------------------------------
// initialize the module with default input
//---------------------------------------------------------------------- 
void INPUTSEL_vInit(uint8_t u8DefaultInput) {
  // we support the input numbers 1...4
  if ( (u8DefaultInput == 0) || (u8DefaultInput > 4u) ) {
	u8DefaultInput = 1u;
  }
  
  // set the default or the saved input
  mg_u8CurrentInput = u8DefaultInput; 

  // set the input
  INPUTSEL_vSet(mg_u8CurrentInput);
}


//----------------------------------------------------------------------
// set the input with the ports
// all outputs have reverse logic
//---------------------------------------------------------------------- 
void INPUTSEL_vSet(uint8_t u8Input) {

  // we support channel 1...4 
  switch (u8Input) {
	case 2u:
	  {
		// save the new input into the module global variable
		mg_u8CurrentInput = u8Input;	
		INPUTSEL_CH1 = 1u;
		INPUTSEL_CH3 = 1u;	
		INPUTSEL_CH4 = 1u;
		// zero means: on
		INPUTSEL_CH2 = 0;	
		break;
	  } 
	case 3u:
	  {
		mg_u8CurrentInput = u8Input;	
		INPUTSEL_CH1 = 1u;
		INPUTSEL_CH2 = 1u;	
		INPUTSEL_CH4 = 1u;
		INPUTSEL_CH3 = 0;
		break;
	  }
	case 4u:
	  {
		mg_u8CurrentInput = u8Input;	
		INPUTSEL_CH1 = 1u;
		INPUTSEL_CH2 = 1u;	
		INPUTSEL_CH3 = 1u;
		INPUTSEL_CH4 = 0;
		break; 
	  } 
	case 1u:
	default:
	  {
		// standard is channel 1
		mg_u8CurrentInput = 1u;	
		INPUTSEL_CH2 = 1u;
		INPUTSEL_CH3 = 1u;	
		INPUTSEL_CH4 = 1u;
		INPUTSEL_CH1 = 0;
		break;
	  } 
  }
}


//----------------------------------------------------------------------
// get the current selected input
//---------------------------------------------------------------------- 
uint8_t INPUTSEL_u8Get(void) {
  return(mg_u8CurrentInput);
}


//----------------------------------------------------------------------
// select the next input channel (function used by the rotary switch)
//----------------------------------------------------------------------
void INPUTSEL_vNextChannel(void) {

  // increase the channel number
  if ( mg_u8CurrentInput < 4u ) {
    mg_u8CurrentInput++;
  } else {
    mg_u8CurrentInput = 1u;
  }

  // set the new input
  INPUTSEL_vSet(mg_u8CurrentInput); 
}


//----------------------------------------------------------------------
// set the relays into standby
//----------------------------------------------------------------------
void INPUTSEL_vSetStandBy(void) {
  // we don't change the current input here,
  // we set only all outputs to high to save the energy for the relays
  INPUTSEL_CH1 = 1u;
  INPUTSEL_CH2 = 1u;
  INPUTSEL_CH3 = 1u;	
  INPUTSEL_CH4 = 1u;
}

