/**********************************************************************
 * Uart1.c
 * Contains a driver for the UART1 debug interface (output only)
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

// include the header of the sdcc compiler
#include <stdint.h>
#include <stdbool.h>

// include our own header
#include "HAL_Settings.h"
#include "Uart1.h"

//#define	UART1_BUF_LENGTH	32u


// counter of sent bytes
//static uint8_t mg_u8Uart1TxCnt;
// counter of received bytes
//volatile uint8_t mg_u8Uart1RxCnt;
// flag signalize: transiver is busy
volatile bool mg_bTxBusy;

//static uint8_t idata mg_au8RxBuf[UART1_BUF_LENGTH];

//----------------------------------------------------------------------
// the main procedure
// ---------------------------------------------------------------------
/*
void main(void)
{
  P0M1 = 0;	P0M0 = 0;
  P1M1 = 0;	P1M0 = 0;
  P2M1 = 0;	P2M0 = 0;
  P3M1 = 0;	P3M0 = 0;
  P4M1 = 0;	P4M0 = 0;
  P5M1 = 0;	P5M0 = 0;
  P6M1 = 0;	P6M0 = 0;
  P7M1 = 0;	P7M0 = 0;

  UART1_vConfig();
  EA = 1;

  UART1_vPrint("STC15F2K60S2 UART1 Test Prgramme!\r\n");

  while (1)
  {
	if((mg_u8Uart1TxCnt != mg_u8Uart1RxCnt) && (!mg_bTxBusy))
	{
	  SBUF = mg_au8RxBuf[mg_u8Uart1TxCnt];
	  mg_bTxBusy = true;
	  if(++mg_u8Uart1TxCnt >= UART1_BUF_LENGTH)	mg_u8Uart1TxCnt = 0;
	}
  }
}
*/

//----------------------------------------------------------------------
// UART1 print function
// ---------------------------------------------------------------------
void UART1_vPrint(char * pcText) {

  // puts until the zero termination  
  while ( *pcText ) {
	// wait until the last byte is written
	while( mg_bTxBusy );
	// we print a character and set the busy flag for it 
	mg_bTxBusy = true;
	// print the character
	SBUF = (uint8_t) (*pcText);
	// prepare the next character
	pcText++; 
  }
}


//----------------------------------------------------------------------
// UART1 setup function (uses Timer2 for baudrate)
// ---------------------------------------------------------------------
void UART1_vConfig(void)	
{
  // calculate the time ticks for the baudrate
  const uint16_t u16TimerReload = (uint16_t) ( 65536u - (UC_u32OscFreq / 4u / UART1_u32Baudrate));

  // USART1 used timer 2 as baudrate generator		
  // we use Timer2 because STC15W408AS has no Timer1
  AUXR |= 0x01;

  // stop Timer2
  AUXR &= ~(1u << 4u);

  // set Timer2 as Timer (and not as counter)
  AUXR &= ~(1u << 3u);

  // set Timer2 into 1 clock mode (not into old 12 clock mode)
  AUXR |= (1u << 2u);

  // Timer 2 reload value
  TH2 = u16TimerReload / 256u;
  TL2 = u16TimerReload % 256u;

  //disable timer 2 interrupt 
  IE2  &= ~(1u << 2u);	  

  // start Timer 2
  AUXR |= (1u << 4u);

  // set the mode (8bit, baudrate selectable)
  SCON = ((SCON & 0x3f) | 0x40);	

  // enable USART1 interrupt
  ES = 1u;

  // enable RX
  //REN = 1u;

  // erase bit 6 and 7 -> uart1 is on P3.0 and P3.1
  P_SW1 &= 0x3f;
  
  // uart1 is on P3.6 and P3.7
  //P_SW1 |= 0x40;

  // disable the relay function
  //PCON2 &=  ~(1u << 4u);

  mg_bTxBusy = false;
//  mg_u8Uart1TxCnt = 0;
//  mg_u8Uart1RxCnt = 0;
}


//----------------------------------------------------------------------
// UART1 interrupt function 
// ---------------------------------------------------------------------
void UART1_int(void) __interrupt(4) {
  
  // check for RX interrupt	
  /*
  if (RI) {
	// erase the RX interrupt flag	
	RI = 0;
	// add the byte to the buffer	
	mg_au8RxBuf[mg_u8Uart1RxCnt] = SBUF;
	// if the buffer full -> start at the begin again	
	if (++mg_u8Uart1RxCnt >= UART1_BUF_LENGTH) {
	  mg_u8Uart1RxCnt = 0;
	}
  }
  */

  // check for TX interrupt	
  if (TI) {
	// erase the TX interrupt flag	
	TI = 0;
	// reset the TX busy flag
	mg_bTxBusy = false;
  }
}

